/************************************************************************
 *
 * Copyright (C) 2006, 2007, 2008, 2010 Graeme Gott <graeme@gottcode.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#ifndef NOVPROG_GRAPH_H
#define NOVPROG_GRAPH_H

#include "data.h"

#include <QCoreApplication>
#include <QGraphicsRectItem>
#include <QGraphicsView>
class QGraphicsScene;

class Bar : public QGraphicsRectItem
{
	Q_DECLARE_TR_FUNCTIONS(Bar)
public:
	Bar(int x, int y, int w, int h, int value, const QDate& day, const QColor& color);

protected:
	void hoverEnterEvent(QGraphicsSceneHoverEvent* e);
	void hoverLeaveEvent(QGraphicsSceneHoverEvent* e);
};


class Graph : public QGraphicsView
{
public:
	Graph(Database* data, Database::GoalType type, QWidget* parent);

	void draw();

private:
	Database* m_data;
	Database::GoalType m_type;
	QGraphicsScene* m_scene;
};

#endif // NOVPROG_GRAPH_H
